package flow.stages

import chisel3._
import flow.Params
import flow.stages.utils._
import flow.stages.messages._
import flow.components._
import flow.components.RV32InstSubfields._
import chisel3.util.DecoupledIO
import chisel3.experimental.Trace._
import chisel3.util.experimental.InlineInstance

class ID(implicit val p: Params) extends Module {
  val io = IO(new Bundle {
    val fromWB = Input(new WB2ID)
  })

  // val msgio = IO(DecoupledMsgIO((new IF2ID).S, (new ID2EX).S))
  val msgio = IO(new Bundle {
    val in = Flipped(DecoupledIO(new IF2ID))
    val out = DecoupledIO(new ID2EX)
  })
  msgio.in.ready := DontCare
  msgio.out.valid := DontCare

  val _in = msgio.in.bits
  val _out = msgio.out.bits
  val _fromWB = io.fromWB

  val regs = Module(new newRegisterFile with InlineInstance)

  // Controllers
  val pcController = Module(new PcController)
  val regFileController = Module(new RegisterFileController)
  val aluController = Module(new AluController)
  val ramController = Module(new RamController)

  pcController.in.inst := _in.inst
  regFileController.in.inst := _in.inst
  aluController.in.inst := _in.inst
  ramController.in.inst := _in.inst

  regs.in.rs1 := _in.inst.rs1
  regs.in.rs2 := _in.inst.rs2
  _out.src1 := regs.out.src1
  _out.src2 := regs.out.src2

  _out.pc := _in.pc
  _out.pcCtrl := pcController.out

  _out.regCtrl := regFileController.out
  _out.aluCtrl := aluController.out
  _out.ramCtrl := ramController.out
  _out.inst := _in.inst

  // register file write back
  import flow.components.RegControl.WriteSelect._
  import flow.components.util.chiselEnumAsInt
  regs.control := _fromWB.regCtrl
  regs.in.rd := _fromWB.rd
  regs.in.writeData(rAluOut) := _fromWB.exeOut
  regs.in.writeData(rMemOut) := _fromWB.memOut
  regs.in.writeData(rNpc) := _fromWB.npc
}
